// For Boolean
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';

class VLB<T> extends StatelessWidget {
  const VLB(
    this.builder, {
    required this.valueListenable,
    super.key,
    this.child,
  });

  final ValueListenable<T> valueListenable;
  final Widget Function(BuildContext context, T value) builder;
  final Widget? child;

  @override
  Widget build(BuildContext context) {
    return ValueListenableBuilder<T>(
      valueListenable: valueListenable,
      builder: (context, value, _) => builder(context, value),
      child: child,
    );
  }
}

class NotifierBool extends ValueNotifier<bool> {
  NotifierBool(super.initial);

  @override
  String toString() => value ? 'true' : 'false';
}

// For Double
class NotifierDouble extends ValueNotifier<double> {
  NotifierDouble(super.initial);

  @override
  String toString() => value.toStringAsFixed(2);
}

// For Integer
class NotifierInt extends ValueNotifier<int> {
  NotifierInt(super.initial);

  @override
  String toString() => value.toString();
}

class NotifierObject extends ValueNotifier<Object> {
  NotifierObject(super.initial);

  @override
  String toString() => value.toString();
}

// For Num (covers both int and double)
class NotifierNum extends ValueNotifier<num> {
  NotifierNum(super.initial);

  @override
  String toString() => value.toString();
}

// For String
class NotifierString extends ValueNotifier<String> {
  NotifierString(super.initial);

  @override
  String toString() => value;
}

// For Color
class NotifierColor extends ValueNotifier<Color> {
  NotifierColor(super.initial);

  @override
  String toString() => value.toString();
}

// For Uri
class NotifierUri extends ValueNotifier<Uri> {
  NotifierUri(super.initial);

  @override
  String toString() => value.toString();
}

// For Generic List<T>
class NotifierList<T> extends ValueNotifier<List<T>> {
  NotifierList(super.initial);

  @override
  String toString() => value.toString();
}

// For Generic Iterable<T>
class NotifierIterable<E> extends ValueNotifier<Iterable<E>> {
  NotifierIterable(super.initial);

  @override
  String toString() => value.toString();
}

// For Generic Map<K, V>
class NotifierMap<K, V> extends ValueNotifier<Map<K, V>> {
  NotifierMap(super.initial);

  @override
  String toString() => value.toString();
}

// For Duration
class NotifierDuration extends ValueNotifier<Duration> {
  NotifierDuration(super.initial);

  @override
  String toString() => value.toString();
}
